@@ -1,4 +1,4 @@
-// Copyright 2019 The TCell Authors
+// Copyright 2020 The TCell Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use file except in compliance with the License.
@@ -58,10 +58,12 @@ type Terminfo struct {
 	Blink        string // blink
 	Reverse      string // rev
 	Dim          string // dim
+	Italic       string // sitm
 	EnterKeypad  string // smkx
 	ExitKeypad   string // rmkx
 	SetFg        string // setaf
 	SetBg        string // setab
+	ResetFgBg    string // op
 	SetCursor    string // cup
 	CursorBack1  string // cub1
 	CursorUp1    string // cuu1
@@ -157,6 +159,8 @@ type Terminfo struct {
 	KeyShfLeft   string // kLFT
 	KeyShfHome   string // kHOM
 	KeyShfEnd    string // kEND
+	KeyShfInsert string // kIC
+	KeyShfDelete string // kDC
 
 	// These are non-standard extensions to terminfo.  This includes
 	// true color support, and some additional keys.  Its kind of bizarre
@@ -164,6 +168,7 @@ type Terminfo struct {
 	// Terminal support for these are going to vary amongst XTerm
 	// emulations, so don't depend too much on them in your application.
 
+	StrikeThrough   string // smxx
 	SetFgBg         string // setfgbg
 	SetFgBgRGB      string // setfgbgrgb
 	SetFgRGB        string // setfrgb
@@ -208,8 +213,16 @@ type Terminfo struct {
 	KeyAltShfEnd    string
 	KeyMetaShfHome  string
 	KeyMetaShfEnd   string
+	Modifiers       int
+	TrueColor       bool // true if the terminal supports direct color
 }
 
+const (
+	ModifiersNone    = 0
+	ModifiersXTerm   = 1
+	ModifiersDynamic = 2
+)
+
 type stackElem struct {
 	s     string
 	i     int
@@ -248,8 +261,17 @@ func (st stack) PopInt() (int, stack) {
 		if e.isInt {
 			return e.i, st
 		} else if e.isStr {
-			i, _ := strconv.Atoi(e.s)
-			return i, st
+			// If the string that was pushed was the representation
+			// of a number e.g. '123', then return the number. If the
+			// conversion doesn't work, assume the string pushed was
+			// intended to return, as an int, the ascii representation
+			// of the (one and only) character.
+			i, err := strconv.Atoi(e.s)
+			if err == nil {
+				return i, st
+			} else if len(e.s) >= 1 {
+				return int(e.s[0]), st
+			}
 		}
 	}
 	return 0, st
@@ -288,13 +310,6 @@ func (st stack) PushBool(i bool) stack {
 	return st.PushInt(0)
 }
 
-func nextch(s string, index int) (byte, int) {
-	if index < len(s) {
-		return s[index], index + 1
-	}
-	return 0, index
-}
-
 // static vars
 var svars [26]string
 
@@ -735,7 +750,9 @@ func LookupTerminfo(name string) (*Terminfo, error) {
 
 	// If the name ends in -truecolor, then fabricate an entry
 	// from the corresponding -256color, -color, or bare terminal.
-	if t == nil && strings.HasSuffix(name, "-truecolor") {
+	if t != nil && t.TrueColor {
+		addtruecolor = true
+	} else if t == nil && strings.HasSuffix(name, "-truecolor") {
 
 		suffixes := []string{
 			"-256color",
