@@ -20,6 +20,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
+	"log"
 	"os"
 	"strconv"
 	"strings"
@@ -49,6 +50,8 @@ const (
 
 	pasteOSC52Begin = "\x1b]52;"
 	pasteOSC52End   = "\x1b\\"
+
+	setTitle = "\x1b]2;title\a"
 )
 
 // NewTerminfoScreen returns a Screen that uses the stock TTY interface
@@ -215,9 +218,24 @@ func (t *tScreen) SetPaste(p bool) {
 }
 
 func (t *tScreen) RegisterRawSeq(r string) {
+	for _, seq := range t.rawseq {
+		if seq == r {
+			return
+		}
+	}
 	t.rawseq = append(t.rawseq, r)
 }
 
+func (t *tScreen) UnregisterRawSeq(r string) {
+	for i, seq := range t.rawseq {
+		if seq == r {
+			t.rawseq[i] = t.rawseq[len(t.rawseq)-1]
+			t.rawseq = t.rawseq[:len(t.rawseq)-1]
+			break
+		}
+	}
+}
+
 func (t *tScreen) prepareKeyMod(key Key, mod ModMask, val string) {
 	if val != "" {
 		// Do not override codes that already exist
@@ -1390,27 +1408,47 @@ func (t *tScreen) parsePaste(buf *bytes.Buffer, evs *[]Event) bool {
 	return false
 }
 
+func (t *tScreen) parseRegisteredRawSeq(buf *bytes.Buffer, evs *[]Event) bool {
+	b := buf.Bytes()
+
+	if b[0] == '\x1b' && len(t.rawseq) > 0 {
+		strb := string(b)
+		for _, r := range t.rawseq {
+			if strings.HasPrefix(strb, r) {
+				// a registered raw sequence matched the prefix
+				*evs = append(*evs, NewEventRaw(r))
+				t.escbuf.Reset()
+				for i := 0; i < len(r); i++ {
+					buf.ReadByte()
+				}
+				return true
+			}
+		}
+	}
+	return false
+}
+
 func (t *tScreen) parseOSC52Paste(buf *bytes.Buffer, evs *[]Event) (bool, bool) {
+	str := buf.String()
 	b := buf.Bytes()
 
 	prefixLen := len(pasteOSC52Begin) + 2
-	suffixLen := len(pasteOSC52End)
-	if bytes.HasPrefix(b, []byte(pasteOSC52Begin)) || bytes.HasPrefix([]byte(pasteOSC52Begin), b) {
-		// OSC52 paste has started
-		if len(b) > len(pasteOSC52Begin)+2 && bytes.HasSuffix(b, []byte(pasteOSC52End)) {
+	if strings.HasPrefix(str, pasteOSC52Begin) || strings.HasPrefix(pasteOSC52Begin, str) {
+		idx := strings.Index(str, pasteOSC52End)
+		log.Println(b, idx, len(str))
+		if idx >= prefixLen {
 			// OSC52 paste has ended
-			payload := b[prefixLen : len(b)-suffixLen]
+			payload := buf.Next(idx)[prefixLen:]
+			buf.Next(len(pasteOSC52End))
 			data := make([]byte, len(payload))
 			n, err := base64.StdEncoding.Decode(data, payload)
 			data = data[:n]
 
 			t.escbuf.Write(b)
 
-			buf.Reset()
-
 			if err != nil {
-				// error must be something else...?
-				return false, false
+				// discard the paste since it is invalid
+				return true, true
 			}
 
 			*evs = append(*evs, NewEventPaste(string(data), t.escbuf.String()))
@@ -1425,20 +1463,17 @@ func (t *tScreen) parseOSC52Paste(buf *bytes.Buffer, evs *[]Event) (bool, bool)
 }
 
 func (t *tScreen) parseBracketedPaste(buf *bytes.Buffer, evs *[]Event) (bool, bool) {
-	b := buf.Bytes()
-
 	// Replace all carriage returns with newlines
-	str := string(bytes.Replace(b, []byte{'\r'}, []byte{'\n'}, -1))
+	str := strings.Replace(buf.String(), "\r", "\n", -1)
 	if strings.HasPrefix(str, pasteBegin) || strings.HasPrefix(pasteBegin, str) {
+		idx := strings.Index(str, pasteEnd)
 		// The bracketed paste has started
-		if strings.HasSuffix(str, pasteEnd) {
+		if idx != -1 && idx >= len(pasteBegin) {
 			// The bracketed paste has ended
 			// Strip out the start and end sequences
-			for i := 0; i < len(b); i++ {
-				by, _ := buf.ReadByte()
-				t.escbuf.WriteByte(by)
-			}
-			*evs = append(*evs, NewEventPaste(str[6:len(b)-6], t.escbuf.String()))
+			t.escbuf.Write(buf.Next(idx + len(pasteEnd)))
+			text := str[len(pasteBegin):idx]
+			*evs = append(*evs, NewEventPaste(text, t.escbuf.String()))
 			t.escbuf.Reset()
 			return true, true
 		}
@@ -1479,6 +1514,10 @@ func (t *tScreen) collectEventsFromInput(buf *bytes.Buffer, expire bool) []Event
 
 		partials := 0
 
+		if t.parseRegisteredRawSeq(buf, &res) {
+			continue
+		}
+
 		if t.paste && t.parsePaste(buf, &res) {
 			continue
 		}
@@ -1526,23 +1565,6 @@ func (t *tScreen) collectEventsFromInput(buf *bytes.Buffer, expire bool) []Event
 
 		if partials == 0 || expire {
 			if b[0] == '\x1b' {
-				strb := string(b)
-				completed := false
-				for _, r := range t.rawseq {
-					if strings.HasPrefix(strb, r) {
-						// a registered raw sequence matched the prefix
-						res = append(res, NewEventRaw(r))
-						t.escbuf.Reset()
-						for i := 0; i < len(r); i++ {
-							buf.ReadByte()
-						}
-						completed = true
-						break
-					}
-				}
-				if completed {
-					continue
-				}
 				if len(b) == 1 {
 					res = append(res, NewEventKey(KeyEsc, 0, ModNone, "\x1b"))
 					t.escbuf.Reset()
@@ -1632,7 +1654,6 @@ func (t *tScreen) inputLoop() {
 		chunk := make([]byte, 4096)
 		n, e := t.in.Read(chunk)
 		switch e {
-		case io.EOF:
 		case nil:
 		default:
 			t.PostEvent(NewEventError(e))
